/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.bpmn.views {
  import flash.display.DisplayObject;
  import flash.events.Event;
  import flash.geom.Point;
  import flash.geom.Rectangle;

  import mx.binding.utils.ChangeWatcher;
  import mx.containers.Canvas;
  import mx.containers.HBox;
  import mx.controls.Label;
  import mx.events.FlexEvent;
  import mx.events.MoveEvent;
  import mx.events.ResizeEvent;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.views.utils.ArrowStyle;
  import org.ow2.orchestra.designer.bpmn.views.utils.ArrowUtil;
  import org.ow2.orchestra.designer.bpmn.views.utils.GraphicUtils;

  public class SequenceFlowView extends AbstractElementView {
    [Bindable]
    private var _flowModel:SequenceFlowModel;

    private var processView:ProcessView;

    private var sourceView:AbstractElementView;
    private var targetView:AbstractElementView;

    private var style:ArrowStyle;
    private var _canvas:Canvas;
    private var conditionLabel:Label;
    private var conditionBox:HBox;

    [Bindable]
    private var _sourcePoint:Point = new Point();
    [Bindable]
    private var _targetPoint:Point = new Point();

    public static const SELECTION_ERROR_MARGIN:Number = 8;

    private var listenerOnSource:Boolean = false;
    private var listenerOnTarget:Boolean = false;

    function SequenceFlowView(flowModel:SequenceFlowModel, processView:ProcessView) {
      this.processView = processView;
      _flowModel = flowModel;
      watchers.addItem(ChangeWatcher.watch(_flowModel, "condition", redraw));
      watchers.addItem(ChangeWatcher.watch(_flowModel, "isDefault", redraw));
      watchers.addItem(ChangeWatcher.watch(_flowModel, "sourceElement", modelChanged));
      watchers.addItem(ChangeWatcher.watch(_flowModel, "targetElement", modelChanged));

      // create canvas:
      _canvas = new Canvas();
      this.addChild(canvas);
      conditionLabel = new Label();
      conditionBox = new HBox();
      conditionBox.setStyle("backgroundColor", "#000000");
      conditionBox.setStyle("color", "#ffffff");
      conditionBox.setStyle("cornerRadius", "10");
      conditionBox.setStyle("borderStyle", "solid");
      conditionBox.addChild(conditionLabel);
      conditionBox.addEventListener(ResizeEvent.RESIZE, setConditionBoxPosition);

      //Set the arrow style
      style = new ArrowStyle();
      style.headWidth = 11;
      style.headLength = 17;
      style.color = 0x000000;
      style.headFill = true;
      style.thickness = 0;

      this.addEventListener(FlexEvent.CREATION_COMPLETE, redraw);
      //Initialize properties
      init();

    }


    private function init(event:Event = null):void {
      sourceView = processView.getElementViewByModel(_flowModel.sourceElement);
      targetView = processView.getElementViewByModel(_flowModel.targetElement);
      if (!sourceView.initialized && !targetView.initialized) {
        this.listenerOnSource = true;
        sourceView.addEventListener(FlexEvent.CREATION_COMPLETE, this.init, false, 0, true);
      } else if (!sourceView.initialized) {
        // Draw the arrow when source view will be created
        this.listenerOnSource = true;
        sourceView.addEventListener(FlexEvent.CREATION_COMPLETE, this.init, false, 0, true);
      } else if (!targetView.initialized) {
        // Draw the arrow when source view will be created
        this.listenerOnTarget = true;
        targetView.addEventListener(FlexEvent.CREATION_COMPLETE, this.init, false, 0, true);
      } else {
        // Draw the arrow
        sourceView.addEventListener(MoveEvent.MOVE, redraw);
        targetView.addEventListener(MoveEvent.MOVE, redraw);
        redraw();
      }
    }

    public override function get model():AbstractElement {
      return _flowModel;
    }

    /**
     * Overrides {@link AbstractElementView#dispose} to remove also {@link FlexEvent.CREATION_COMPLETE} events on source and
     * target views.
     */
    override public function dispose():void {
      super.dispose();
      if (this.listenerOnSource) {
        var sourceView:AbstractElementView = processView.getElementViewByModel(_flowModel.sourceElement);
        sourceView.removeEventListener(FlexEvent.CREATION_COMPLETE, this.init);
      }
      if (this.listenerOnTarget) {
        var targetView:AbstractElementView = processView.getElementViewByModel(_flowModel.targetElement);
        targetView.removeEventListener(FlexEvent.CREATION_COMPLETE, this.init);
      }
      if (sourceView != null) {
        sourceView.removeEventListener(MoveEvent.MOVE, redraw);
      }
      if (targetView != null) {
        targetView.removeEventListener(MoveEvent.MOVE, redraw);
      }
    }

    private function modelChanged(event:Event = null):void {
      if (sourceView != null) {
        sourceView.removeEventListener(MoveEvent.MOVE, redraw);
      }
      if (targetView != null) {
        targetView.removeEventListener(MoveEvent.MOVE, redraw);
      }
      sourceView = processView.getElementViewByModel(_flowModel.sourceElement);
      targetView = processView.getElementViewByModel(_flowModel.targetElement);
      sourceView.addEventListener(MoveEvent.MOVE, redraw);
      targetView.addEventListener(MoveEvent.MOVE, redraw);
      redraw();
    }

    public function redraw(event:Event = null):void {
      style.isConditional = _flowModel.condition != null && _flowModel.condition != "";
      style.isDefault = _flowModel.isDefault;


      // get source and target border image
      var sourceImage:DisplayObject = sourceView.borderImage;
      var targetImage:DisplayObject = targetView.borderImage;

      // get image centers
      var sourceCenterPoint:Point = sourceView.localToGlobal(new Point(sourceImage.x
              + (sourceImage.width / 2), sourceImage.y + (sourceImage.height / 2)));
      var targetCenterPoint:Point = targetView.localToGlobal(new Point(targetImage.x
              + (targetImage.width / 2), targetImage.y + (targetImage.height / 2)));

      // update source and target points
      _sourcePoint = this.globalToLocal(GraphicUtils.findIntersection(sourceImage, sourceCenterPoint, targetCenterPoint, processView));
      _targetPoint = this.globalToLocal(GraphicUtils.findIntersection(targetImage, targetCenterPoint, sourceCenterPoint, processView));

      // draw arrow
      ArrowUtil.drawArrow(canvas.graphics, _sourcePoint, _targetPoint, style);

      // draw condition
      if (style.isConditional) {
        conditionLabel.text = _flowModel.condition;
        conditionBox.setVisible(true);
        setConditionBoxPosition();
        canvas.removeAllChildren();
        canvas.addChild(conditionBox);
      } else {
        canvas.removeAllChildren();
      }
    }

    private function setConditionBoxPosition(event:Event = null):void {
      conditionBox.x = (_sourcePoint.x + _targetPoint.x - conditionBox.width) / 2;
      conditionBox.y = (_sourcePoint.y + _targetPoint.y - conditionBox.height) / 2;
    }

    public function set sourcePoint(value:Point):void {
      _sourcePoint = value;
      ArrowUtil.drawArrow(canvas.graphics, _sourcePoint, _targetPoint, style);
    }

    public function set targetPoint(value:Point):void {
      _targetPoint = value;
      ArrowUtil.drawArrow(canvas.graphics, _sourcePoint, _targetPoint, style);
    }

    public function get sourcePoint():Point {
      return _sourcePoint;
    }

    public function get targetPoint():Point {
      return _targetPoint;
    }

    /**
     * Overrides the {@link AbstractElementView#includeInto} to check if target and source points are in the selection
     * rectangle.
     * @param selection A {@link Rectangle} representing the selection
     * @param targetCoordinateSpace The {@link DisplayObject} from which <code>selection</code> coordinates are relative
     * @return Returns true if the selection rectangle is valid to select the object false otherwise
     */
    override public function includeInto(selection:Rectangle, targetCoordinateSpace:DisplayObject):Boolean {
      //TODO: generalize to any kind of elements containing arrows
      if (selection.containsPoint(this.sourcePoint) && selection.containsPoint(this.targetPoint)) {
        return true;
      }

      return false;
    }

    /**
     * Overrides the {@link AbstractElementView#containsRectangle} to check if any corner of the selection rectangle is
     * separated from the arrow line by less than {@link SequenceFlowView#SELECTION_ERROR_MARGIN}
     *
     * @param selection A {@link Rectangle} representing the selection
     * @param targetCoordinateSpace The {@link DisplayObject} from which <code>selection</code> coordinates are relative
     * @return Returns true if the selection rectangle is valid to select the object false otherwise
     */
    override public function containsRectangle(selection:Rectangle, targetCoordinateSpace:DisplayObject):Boolean {
      //TODO: generalize to any kind of elements containing arrows
      var topLeftCornerDistance:Number = GraphicUtils.distanceToSegment(this.sourcePoint, this.targetPoint, selection.topLeft);
      var bottomLeftCornerDistance:Number = GraphicUtils.distanceToSegment(this.sourcePoint, this.targetPoint, new Point(selection.x,
              selection.y + selection.width));
      var topRightCornerDistance:Number = GraphicUtils.distanceToSegment(this.sourcePoint, this.targetPoint, new Point(selection.x
              + selection.height,
              selection.y));
      var bottomRightCornerDistance:Number = GraphicUtils.distanceToSegment(this.sourcePoint, this.targetPoint, new Point(selection.x
              + selection.height,
              selection.y + selection.width));

      if (topLeftCornerDistance <= SELECTION_ERROR_MARGIN && bottomLeftCornerDistance <= SELECTION_ERROR_MARGIN && topRightCornerDistance <= SELECTION_ERROR_MARGIN
              && bottomRightCornerDistance <= SELECTION_ERROR_MARGIN) {
        return true;
      }

      return false;
    }

    public function get canvas():Canvas {
      return _canvas;
    }

  }
}
